/*
 * ServletControlador.java
 *
 * Created on 3 de mayo de 2007, 18:54
 */

package mvc;

import java.io.*;
import java.util.*; 
import javax.servlet.*; 
import javax.servlet.http.*; 

public class ServletControlador extends HttpServlet { 
    //Variables obtenidas del Descriptor de Despliegue (DD) 
    private static final String INICIALIZADOR = "inicializador"; 
    private static final String PREFIJO_EVENTO = "evento."; 
    // Otras variables 
    private static final String NOM_TABLA_EVENTOS = "tablaEventos"; 
    
    //Inicializacin del servlet 
    public void init(ServletConfig config) throws ServletException { 
        super.init(config); 
        // Util para no perderse en los logs
        mostrar(""); 
        mostrar("********* Comienza prctica 5 **********"); 
        try { 
        // Recuperar la clase inicializadora de la aplicacin (ver DD). 
            String inicializador = config.getInitParameter(INICIALIZADOR); 
            Inicializador ini = (Inicializador) Class.forName(inicializador).newInstance();
            ini.init(config); 
            // Recuperar las clases de los eventos 
            Map eventos = new HashMap(); 
            Enumeration e = config.getInitParameterNames(); 
            // Recorrer los parmetros de inicio buscando eventos 
            while (e.hasMoreElements()) { 
                String nombre = (String)e.nextElement(); 
                if (nombre.startsWith(PREFIJO_EVENTO)) { 
                //Es un evento 
                    String clase = config.getInitParameter(nombre);
                    //Clase que maneja el evento 
                    Evento evento = (Evento)Class.forName(clase).newInstance(); 
                    eventos.put(nombre, evento); 
                    mostrar("Clase " + clase + " registrada para eventos de tipo " + nombre); 
                } 
            }
            //Guardar la tabla de eventos en el contexto 
            config.getServletContext().setAttribute( NOM_TABLA_EVENTOS, eventos); 
        } catch (Exception ex) { 
            mostrar(ex.getMessage()); 
            ex.printStackTrace(); 
            throw new UnavailableException(ex.getMessage()); 
        } 
    } 
    public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException { 
        // Busca en la estructura HTTP el parmetro y el valor que se lanzan 
        // desde el formulario
        String nomEvento = req.getParameter(Evento.NOM_EVENTO);
        // Obtiene la coleccin de eventos definidos 
        Map eventos = (Map)getServletContext().getAttribute(NOM_TABLA_EVENTOS); 
        if (!eventos.containsKey(nomEvento)) { 
            //Si no se encuentra el evento se lanza excepcin 
            String msg = "Evento no encontrado " + nomEvento; mostrar(msg); 
            throw new ServletException(msg); 
        }
        else { 
            // Si se encuentra el evento se procesa 
            mostrar("Procesando evento " + nomEvento); 
            // Se recupera la clase controladora 
            Evento evento = (Evento)eventos.get(nomEvento); 
            // Se delega el evento del proceso 
            String path = evento.procesar(getServletContext(), req); 
            mostrar("Evento procesado, redirigiendo a " + path); 
            // Se redirige la peticin 
            req.getRequestDispatcher(path).forward(req, res); 
            mostrar("Evento " + nomEvento + " procesado"); 
        } 
    } 
    private void mostrar(String msg) { 
        System.out.println(msg); 
        //log(msg); 
    } 
} 
